@@ -2,6 +2,7 @@ from .common import InfoExtractor
 
 
 class RestudyIE(InfoExtractor):
+    _WORKING = False
     _VALID_URL = r'https?://(?:(?:www|portal)\.)?restudy\.dk/video/[^/]+/id/(?P<id>[0-9]+)'
     _TESTS = [{
         'url': 'https://www.restudy.dk/video/play/id/1637',
@@ -14,7 +15,7 @@ class RestudyIE(InfoExtractor):
         'params': {
             # rtmp download
             'skip_download': True,
-        }
+        },
     }, {
         'url': 'https://portal.restudy.dk/video/leiden-frosteffekt/id/1637',
         'only_matching': True,
@@ -29,7 +30,7 @@ class RestudyIE(InfoExtractor):
         description = self._og_search_description(webpage).strip()
 
         formats = self._extract_smil_formats(
-            'https://cdn.portal.restudy.dk/dynamic/themes/front/awsmedia/SmilDirectory/video_%s.xml' % video_id,
+            f'https://cdn.portal.restudy.dk/dynamic/themes/front/awsmedia/SmilDirectory/video_{video_id}.xml',
             video_id)
 
         return {
