@@ -1,7 +1,7 @@
 import os
+import sys
 
 from .common import PostProcessor
-from ..compat import compat_os_name
 from ..utils import (
     PostProcessingError,
     XAttrMetadataError,
@@ -34,8 +34,17 @@ class XAttrMetadataPP(PostProcessor):
         # (e.g., 4kB on ext4), and we don't want to have the other ones fail
         'user.dublincore.description': 'description',
         # 'user.xdg.comment': 'description',
+        'com.apple.metadata:kMDItemWhereFroms': 'webpage_url',
     }
 
+    APPLE_PLIST_TEMPLATE = '''<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<array>
+\t<string>%s</string>
+</array>
+</plist>'''
+
     def run(self, info):
         mtime = os.stat(info['filepath']).st_mtime
         self.to_screen('Writing metadata to file\'s xattrs')
@@ -45,6 +54,11 @@ class XAttrMetadataPP(PostProcessor):
                 if value:
                     if infoname == 'upload_date':
                         value = hyphenate_date(value)
+                    elif xattrname == 'com.apple.metadata:kMDItemWhereFroms':
+                        # Colon in xattr name throws errors on Windows/NTFS and Linux
+                        if sys.platform != 'darwin':
+                            continue
+                        value = self.APPLE_PLIST_TEMPLATE % value
                     write_xattr(info['filepath'], xattrname, value.encode())
 
             except XAttrUnavailableError as e:
@@ -57,7 +71,7 @@ class XAttrMetadataPP(PostProcessor):
                 elif e.reason == 'VALUE_TOO_LONG':
                     self.report_warning(f'Unable to write extended attribute "{xattrname}" due to too long values.')
                 else:
-                    tip = ('You need to use NTFS' if compat_os_name == 'nt'
+                    tip = ('You need to use NTFS' if os.name == 'nt'
                            else 'You may have to enable them in your "/etc/fstab"')
                     raise PostProcessingError(f'This filesystem doesn\'t support extended attributes. {tip}')
 
